#!/bin/bash
TMP_DIR="/dev/shm/install"

set -e

unmount_if_needed() {
	if mount | grep /dev/mmcblk0p1; then
		umount -f /dev/mmcblk0p1
	fi

	if mount | grep /dev/mmcblk0p2; then
		umount -f /dev/mmcblk0p2
	fi

	if mount | grep /dev/mmcblk0p3; then
		umount -f /dev/mmcblk0p3
	fi

	if mount | grep /dev/mmcblk0p4; then
		umount -f /dev/mmcblk0p4
	fi
}

do_exit() {
	unmount_if_needed
}

trap do_exit EXIT

ls -l $1

# Check if the second argument is 'A' or 'B'.  This determines if you are
# installing to the A side or B side, and is used to make a variety of
# decisions during the install process.
if [ "$2" != "A" ] && [ "$2" != "B" ]; then
	echo "Must specify if this is an A or B side install!"; exit 1
else
	echo "Installing to $2 side."
fi

if [ "$(id -u)" != "0"  ]; then
	echo "Install script must be run as root user or with sudo!"; exit 1
fi

echo "Creating workspace and unpacking firmware..."

if [ -d $TMP_DIR ]; then
	rm -rf $TMP_DIR
fi

mkdir -p $TMP_DIR
tar -C $TMP_DIR -xmf $1
tar -C $TMP_DIR -xmf "$TMP_DIR/firmware.bin"

unmount_if_needed

echo "Decrypting packages..."
gpg -o $TMP_DIR/u-boot.tar.xz --skip-verify -d $TMP_DIR/u-boot.tar.xz.gpg
rm $TMP_DIR/u-boot.tar.xz.gpg
gpg -o $TMP_DIR/rootfs.tar.xz --skip-verify -d $TMP_DIR/rootfs.tar.xz.gpg
rm $TMP_DIR/rootfs.tar.xz.gpg
gpg -o $TMP_DIR/code-a.tar.xz --skip-verify -d $TMP_DIR/code-a.tar.xz.gpg
rm $TMP_DIR/code-a.tar.xz.gpg
gpg -o $TMP_DIR/code-b.tar.xz --skip-verify -d $TMP_DIR/code-b.tar.xz.gpg
rm $TMP_DIR/code-b.tar.xz.gpg
gpg -o $TMP_DIR/modules.tar.xz --skip-verify -d $TMP_DIR/modules.tar.xz.gpg
rm $TMP_DIR/modules.tar.xz.gpg
gpg -o $TMP_DIR/gpg.tar.xz --skip-verify -d $TMP_DIR/gpg.tar.xz.gpg
rm $TMP_DIR/gpg.tar.xz.gpg
echo "Done decrypting packages."

echo "Removing existing partition table and boot sector..."
dd if=/dev/zero of=/dev/mmcblk0 bs=512 count=2048
sync
echo "Done removing existing partition table and boot sector."

echo "Writing u-boot to disk..."
tar -C $TMP_DIR -xmf $TMP_DIR/u-boot.tar.xz
dd if=$TMP_DIR/u-boot.bin of=/dev/mmcblk0 bs=1K seek=33
sync
echo "Done writing u-boot..."

echo "Creating partition table..."
sfdisk --force /dev/mmcblk0 <  $TMP_DIR/csi.part
echo "Done creating partition table."

echo "Trying to make sure the kernel has up to date info and that new partitions aren't mounted..."
partx -u /dev/mmcblk0
sleep 5
unmount_if_needed
echo "Done waiting."

echo "Formatting partitions..."
mkfs.ext4 -F -F /dev/mmcblk0p1
mkfs.ext4 -F -F /dev/mmcblk0p2
mkfs.ext4 -F -F /dev/mmcblk0p3
mkfs.ext4 -F -F /dev/mmcblk0p4
echo "Done formatting partitions."

echo "Setting up boot partition..."
mkdir -p /mnt/boot
mount /dev/mmcblk0p1 /mnt/boot
cp $TMP_DIR/uEnv.1 /mnt/boot/
cp $TMP_DIR/uEnv.2 /mnt/boot/
ln -s uEnv.1 /mnt/boot/uEnv.txt
cp $TMP_DIR/u-boot.sha256 /mnt/boot/u-boot.sha256
echo "Done setting up boot partition."

echo "Setting up root partitions..."

mkdir -p /mnt/root1
mkdir -p /mnt/root2

mount /dev/mmcblk0p2 /mnt/root1
mount /dev/mmcblk0p3 /mnt/root2

tar -C /mnt/root1 -xmf $TMP_DIR/rootfs.tar.xz > /dev/null
tar -C /mnt/root2 -xmf $TMP_DIR/rootfs.tar.xz > /dev/null

ln -sf fstab.1 /mnt/root1/etc/fstab
ln -sf fstab.2 /mnt/root2/etc/fstab

# Setup meta
echo -n "stable" > /mnt/root1/csi/meta/status
echo -n "stable" > /mnt/root2/csi/meta/status
echo -n 1 > /mnt/root1/csi/meta/partition_id
echo -n 2 > /mnt/root2/csi/meta/partition_id
echo -n 1 > /mnt/root1/csi/meta/partition_id
echo -n 2 > /mnt/root2/csi/meta/partition_id
echo -n $2 > /mnt/root1/csi/meta/board
echo -n $2 > /mnt/root2/csi/meta/board

cp $TMP_DIR/rootfs.sha256 /mnt/root1/csi/meta/rootfs.sha256
cp $TMP_DIR/rootfs.sha256 /mnt/root2/csi/meta/rootfs.sha256

cp $TMP_DIR/modules.sha256 /mnt/root1/csi/meta/modules.sha256
cp $TMP_DIR/modules.sha256 /mnt/root2/csi/meta/modules.sha256


if [ $2 = "A" ]; then
    echo "Handling A side..."

	DEFAULT_IP='192.168.5.40\/24'

	cp $TMP_DIR/code-a.sha256 /mnt/root1/csi/meta/code.sha256
	cp $TMP_DIR/code-a.sha256 /mnt/root2/csi/meta/code.sha256
	tar -xmf $TMP_DIR/code-a.tar.xz -C /mnt/root1/csi/code/
	tar -xmf $TMP_DIR/code-a.tar.xz -C /mnt/root2/csi/code/
fi

if [ $2 = "B" ]; then
	echo "Handling B side..."

	DEFAULT_IP='192.168.5.41\/24'

	cp $TMP_DIR/code-b.sha256 /mnt/root1/csi/meta/code.sha256
	cp $TMP_DIR/code-b.sha256 /mnt/root2/csi/meta/code.sha256
	tar -xmf $TMP_DIR/code-b.tar.xz -C /mnt/root1/csi/code/
	tar -xmf $TMP_DIR/code-b.tar.xz -C /mnt/root2/csi/code/
fi

# Extract necessary modules
tar xmf $TMP_DIR/modules.tar.xz -C /mnt/root1/
tar xmf $TMP_DIR/modules.tar.xz -C /mnt/root2/

# Install modules
ls -l /mnt/root1/lib/modules
ls -l /mnt/root2/lib/modules/
KERNEL_VERSION=`ls /mnt/root1/lib/modules/`
depmod -b /mnt/root1 -a $KERNEL_VERSION
depmod -b /mnt/root2 -a $KERNEL_VERSION

# Blacklist network driver if B side and not a DEV version
if [ $2 = "B" ] && [ ! -f /mnt/root1/csi/meta/dev ]; then
	echo "blacklist fec" >> /mnt/root1/etc/modprobe.d/variscite-blacklist.conf
	echo "blacklist fec" >> /mnt/root2/etc/modprobe.d/variscite-blacklist.conf
fi

# If this is dev code...
if [ -f /mnt/root1/csi/meta/dev ]; then
	ln -s imx8mq-var-dart-fre-csi-dev.dtb /mnt/root1/boot/imx8mq-var-dart-fre-csi.dtb
	ln -s imx8mq-var-dart-fre-csi-dev.dtb /mnt/root2/boot/imx8mq-var-dart-fre-csi.dtb
elif [ $2 = "A" ]; then
	ln -s imx8mq-var-dart-fre-csi-a.dtb /mnt/root1/boot/imx8mq-var-dart-fre-csi.dtb
	ln -s imx8mq-var-dart-fre-csi-a.dtb /mnt/root2/boot/imx8mq-var-dart-fre-csi.dtb
elif [ $2 = "B" ]; then
	ln -s imx8mq-var-dart-fre-csi-b.dtb /mnt/root1/boot/imx8mq-var-dart-fre-csi.dtb
	ln -s imx8mq-var-dart-fre-csi-b.dtb /mnt/root2/boot/imx8mq-var-dart-fre-csi.dtb
fi

# Setting up GPG
tar -C /mnt/root1/home/root/ -xmf $TMP_DIR/gpg.tar.xz
tar -C /mnt/root2/home/root/ -xmf $TMP_DIR/gpg.tar.xz
cp $TMP_DIR/gpg.sha256 /mnt/root1/csi/meta/gpg.sha256
cp $TMP_DIR/gpg.sha256 /mnt/root2/csi/meta/gpg.sha256

echo "Setting permissions..."
# Set permissions
chmod 500 /mnt/root1/csi/code/scripts/*
chmod 500 /mnt/root2/csi/code/scripts/*

echo "Done setting up root partitions."



echo "Setting up var partition..."

if [ ! -d /mnt/var ]; then
	mkdir /mnt/var
fi

unmount_if_needed
mount /dev/mmcblk0p4 /mnt/var

mkdir -p /mnt/var/log
mkdir -p /mnt/var/config
mkdir -p /mnt/var/firmware
mkdir -p /mnt/var/keys

echo "Done setting up var partition."
echo "Cleaning up..."
sync
echo "Done cleaning up."
echo "Install complete."
